if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_Arrow requires PitBull4")
end

-- CONSTANTS ----------------------------------------------------------------

local UI_TEXTURE = [[Interface\Minimap\MinimapArrow]]

local CONTAINER_WIDTH = 32
local CONTAINER_HEIGHT = 32
local RANGE_DISTANCE = 42

-----------------------------------------------------------------------------

local L = PitBull4.L

local PitBull4_Arrow = PitBull4:NewModule("Arrow", "AceEvent-3.0")

PitBull4_Arrow:SetModuleType("indicator")
PitBull4_Arrow:SetName(L["Arrow"])
PitBull4_Arrow:SetDescription(L["Shows an arrow that points to other players in a raid."])
PitBull4_Arrow:SetDefaults({
	attach_to = "root",
	location = "out_top",
	position = 1,
	size = 1.5,
	only_ranged = true,
	show_distance = true,
	distance_color = { 1, 1, 0, 1 },
	update_frequency = 0,
	vertical_offset = 0,
	horizontal_offset = 0,
	vertical_text_offset = 0,
	horizontal_text_offset = 0,
})

local mapfiles = LibStub("LibMapData-1.0")
local width = 100
local height = 100

local s2 = sqrt(2)
local cos, sin, rad, atan2, sqrt = math.cos, math.sin, math.rad, math.atan2, math.sqrt
local strfind = string.find
local GetPlayerFacing, GetPlayerMapPosition = GetPlayerFacing, GetPlayerMapPosition

local function CalculateCorner (r)
	return 0.5 + cos(r) / s2, 0.5 + sin(r) / s2;
end

local function RotateTexture (texture, angle)
	local LRx, LRy = CalculateCorner (angle + 0.785398163);
	local LLx, LLy = CalculateCorner (angle + 2.35619449);
	local ULx, ULy = CalculateCorner (angle + 3.92699082);
	local URx, URy = CalculateCorner (angle - 0.785398163);
	texture:SetTexCoord (ULx, ULy, LLx, LLy, URx, URy, LRx, LRy);
end

local function badUnit (frame)
	local unit = frame.unit
	if
		not unit
		or UnitIsUnit (unit, "player")
		or (
			not UnitPlayerOrPetInRaid (unit)
			and not UnitPlayerOrPetInParty (unit)
		) then
		return true
	end
	return false
end

local function badUnitCheck (frame)
	local container = frame.Arrow
	if badUnit (frame) then
		container.arrow:Hide()
		container.text:Hide()
		return true
	end
	container.arrow:Show()
	if container.db.show_distance then
 		container.text:Show()
	end
	return false
end

local function resetAllFrames (that)
	for frame in PitBull4:IterateFrames() do
		that:Clear(frame)
		that:Update(frame)
	end
end

local function OnUpdate (self, elapsed)
	self.elapsed = self.elapsed + elapsed
	local frame = self.frame
	if badUnitCheck (frame) then
		return
	end
	if not self.arrow then
		return
	end
	if (self.elapsed > self.db.update_frequency) then
		self.elapsed = 0
		local tx, ty = GetPlayerMapPosition (self.unit)
		if tx == 0 and ty == 0 then
			self.arrow:Hide()
			self.text:Hide()
			return
		end
		local px, py = GetPlayerMapPosition ("player")
		local distance = sqrt ((((tx - px) * width) ^ 2) + (((ty - py) * height) ^ 2))
		if self.db.only_ranged and distance < RANGE_DISTANCE then
			self.arrow:Hide()
			self.text:Hide()
			return
		end
		self.arrow:Show()
		if self.db.show_distance then
			self.text:Show()
			self.text:SetText(format ("%d ", distance))
		else
			self.text:Hide()
		end
		local bearing = -GetPlayerFacing () - atan2 (tx - px, py - ty)
		RotateTexture (self.arrow, bearing)
	end
end

function PitBull4_Arrow:OnEnable()
	mapfiles:RegisterCallback("MapChanged", function (event, map, floor, w, h)
		width = w
		height = h
	end)
end

function PitBull4_Arrow:ClearFrame(frame)
	local container = frame.Arrow
	if not container then
		return false
	end
	container:SetScript ("OnUpdate", nil)
	container.arrow = container.arrow:Delete()
	container.text = container.text:Delete()
	frame.Arrow = container:Delete()
	return true
end

function PitBull4_Arrow:UpdateFrame(frame)
	if badUnit (frame) then
		return self:ClearFrame(frame)
	end
	local container = frame.Arrow
	local class_db = frame.classification_db
	local xmultiplier = 1
	local ymultiplier = 1
	if class_db then
		if class_db.horizontal_mirror then
			xmultiplier = -1
		end
		if class_db.vertical_mirror then
			ymultiplier = -1
		end
	end
	if not container then
		print ("Creating container for " .. frame.unit)
		container = PitBull4.Controls.MakeFrame(frame)
		container.frame = frame
		container.elapsed = 0
		local db = self:GetLayoutDB(frame)
		container.db = db
		container.unit = frame.unit
		frame.Arrow = container
		container:SetWidth(CONTAINER_WIDTH)
		container:SetHeight(CONTAINER_HEIGHT)
		container:SetScale(db.size * 2)
		container:SetFrameLevel(frame:GetFrameLevel() + 20)
		local arrow = PitBull4.Controls.MakeTexture(container, "BACKGROUND")
		container.arrow = arrow
		arrow:SetTexture(UI_TEXTURE)
		arrow:SetAllPoints(container)
		arrow:ClearAllPoints()
		arrow:SetPoint("CENTER", db.horizontal_offset * xmultiplier, -db.vertical_offset * ymultiplier)
		local text = PitBull4.Controls.MakeFontString(container, "OVERLAY")
		container.text = text
		text:SetShadowColor(0, 0, 0, 1)
		text:SetShadowOffset(0.8, -0.8)
		text:SetPoint("CENTER", container, "CENTER", db.horizontal_text_offset * xmultiplier, -db.vertical_text_offset * ymultiplier)
		text:SetTextColor(unpack (db.distance_color))
		local font, fontsize = self:GetFont(frame)
		text:SetFont(font, fontsize, "OUTLINE")
		container:SetScript ("OnUpdate", OnUpdate)
	end
	container:Show()
	return true
end

PitBull4_Arrow:SetLayoutOptionsFunction(function(self)
	return
	'only_ranged', {
		type = 'toggle',
		name = L["Only show ranged units"],
		desc = L["The arrow only appears when a unit has gone out of range."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).only_ranged
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).only_ranged = value
			resetAllFrames (self)
		end,
		order = 100,
	},
	'update_frequency', {
		type = 'range',
		name = L["Update frequency"],
		desc = L["How often the arrow updates. (Use higher values for slower CPUs)"],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).update_frequency
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).update_frequency = value
			resetAllFrames (self)
		end,
		softMin = 0,
		softMax = 1,
		step = 0.05,
		order = 101,
	},
	'show_distance', {
		type = 'toggle',
		name = L["Show distance"],
		desc = L["Shows the distance to the raid member as text."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).show_distance = value
			resetAllFrames (self)
		end,
		order = 102,
	},
	'distance_color', {
		type = 'color',
		hasAlpha = true,
		name = L["Distance text color"],
		desc = L["The color of the distance text."],
		get = function(info)
			return unpack(PitBull4.Options.GetLayoutDB(self).distance_color)
		end,
		set = function(info, r, g, b, a)
			local color = PitBull4.Options.GetLayoutDB(self).distance_color
			color[1], color[2], color[3], color[4] = r, g, b, a
			resetAllFrames (self)
		end,
		hidden = function ()
			return not PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		order = 103,
	},
	'vertical_offset', {
		type = 'range',
		name = L["Vertical offset"],
		desc = L["Adjust exactly where the arrow appears on the vertical axis."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).vertical_offset
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).vertical_offset = value
			resetAllFrames (self)
		end,
		hidden = function ()
			return not PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		softMin = -30,
		softMax = 30,
		step = 1,
		order = 104,
	},
	'horizontal_offset', {
		type = 'range',
		name = L["Horizontal offset"],
		desc = L["Adjust exactly where the arrow appears on the horizontal axis."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).horizontal_offset
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).horizontal_offset = value
			resetAllFrames (self)
		end,
		hidden = function ()
			return not PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		softMin = -30,
		softMax = 30,
		step = 1,
		order = 105,
	},
	'vertical_text_offset', {
		type = 'range',
		name = L["Vertical text offset"],
		desc = L["Adjust the distance text vertically relative to the arrow."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).vertical_text_offset
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).vertical_text_offset = value
			resetAllFrames (self)
		end,
		hidden = function ()
			return not PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		softMin = -30,
		softMax = 30,
		step = 1,
		order = 106,
	},
	'horizontal_text_offset', {
		type = 'range',
		name = L["Horizontal text offset"],
		desc = L["Adjust the distance text horizontally relative to the arrow."],
		get = function(info)
			return PitBull4.Options.GetLayoutDB(self).horizontal_text_offset
		end,
		set = function(info, value)
			PitBull4.Options.GetLayoutDB(self).horizontal_text_offset = value
			resetAllFrames (self)
		end,
		hidden = function ()
			return not PitBull4.Options.GetLayoutDB(self).show_distance
		end,
		softMin = -30,
		softMax = 30,
		step = 1,
		order = 107,
	}
end)
